@@ -37,7 +37,11 @@ from . import Mlx90614
 from . import WitMotion
 from . import Sensirion
 from . import GCode
+from . import Pca9685
+from . import Omron
 from . import Tongmen  # require pymodbus
 from . import Zhouzheng  # require pymodbus
 from . import Miaoguan  # require pymodbus
+from . import Sako  # require pymodbus
+
 
